@@ -57,7 +57,6 @@ static struct file_acl *get_a_matching_object(struct proc_acl *subject,
 struct file_acl *get_matching_object(struct proc_acl *subject, const char *filename)
 {
 	struct file_acl *tmpf = NULL;
-	struct file_acl *tmpg = NULL;
 	char *tmpname = alloca(strlen(filename) + 1);
 
 	strcpy(tmpname, filename);
@@ -241,6 +240,27 @@ check_default_objects(struct role_acl *role)
 	return;
 }
 
+static void
+check_subject_modes(struct role_acl *role)
+{
+	struct proc_acl *tmp;
+	struct file_acl *tmpf;
+
+	for_each_subject(tmp, role) {
+		if ((tmp->mode & GR_LEARN) && (tmp->mode & GR_INHERITLEARN)) {
+			fprintf(stderr, "Invalid subject mode found for "
+				"role %s subject %s\nBoth \"i\" and \"l\" modes "
+				"cannot be used together.  Please choose either "
+				"normal or inheritance-based learning for the "
+				"subject.\nThe RBAC system will not load until you "
+				"correct this error.\n", role->rolename, tmp->filename);
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	return;
+}
+
 static void
 check_socket_policies(struct role_acl *role)
 {
@@ -596,6 +616,7 @@ analyze_acls(void)
 		}
 
 		check_default_objects(role);
+		check_subject_modes(role);
 		check_socket_policies(role);
 
 		/* non-critical warnings aren't issued for special roles */
@@ -770,6 +791,14 @@ analyze_acls(void)
 			errs_found++;
 		}
 
+		if (!stat("/libx32", &fstat) && !check_permission(role, def_acl, "/libx32", &chk)) {
+			fprintf(stderr,
+				"Write access is allowed by role %s to /libx32, a directory which "
+				"holds system libraries.\n\n",
+				role->rolename);
+			errs_found++;
+		}
+
 		if (!stat("/usr/lib32", &fstat) && !check_permission(role, def_acl, "/usr/lib32", &chk)) {
 			fprintf(stderr,
 				"Write access is allowed by role %s to /usr/lib32, a directory which "
@@ -777,6 +806,13 @@ analyze_acls(void)
 			errs_found++;
 		}
 
+		if (!stat("/usr/libx32", &fstat) && !check_permission(role, def_acl, "/usr/libx32", &chk)) {
+			fprintf(stderr,
+				"Write access is allowed by role %s to /usr/libx32, a directory which "
+				"holds system libraries.\n\n", role->rolename);
+			errs_found++;
+		}
+
 
 		if (!stat("/lib64", &fstat) && !check_permission(role, def_acl, "/lib64", &chk)) {
 			fprintf(stderr,
