@@ -1,5 +1,9 @@
 #include "gradm.h"
 
+struct glob_file *glob_files_head;
+struct glob_file *glob_files_tail;
+struct symlink *symlinks;
+struct proc_acl *global_nested_subject_list;
 struct deleted_file *deleted_files;
 struct role_acl *current_role;
 struct proc_acl *current_subject;
@@ -22,5 +26,11 @@ u_int16_t current_learn_rolemode;
 char **dont_reduce_dirs;
 char **always_reduce_dirs;
 char **protected_paths;
+char **read_protected_paths;
 char **high_reduce_dirs;
 char **high_protected_paths;
+u_int32_t grlearn_options;
+
+char *output_log;
+
+char *learn_log_buffer;
