@@ -126,6 +126,22 @@ transmit_to_kernel(struct gr_arg_wrapper *buf)
 	return err;
 }
 
+void ctrl_sighandler(int sig)
+{
+	struct termios term;
+	signal(sig, SIG_IGN);
+
+	tcgetattr(STDIN_FILENO, &term);
+	term.c_lflag |= ECHO;
+	tcsetattr(STDIN_FILENO, TCSANOW, &term);
+
+	printf("\n");
+	fflush(stdout);
+
+	ioctl(0, TIOCNXCL);
+	exit(EXIT_FAILURE);
+}
+
 void check_acl_status(u_int16_t reqmode)
 {
 	int fd;
@@ -133,6 +149,7 @@ void check_acl_status(u_int16_t reqmode)
 	struct gr_arg arg;
 	struct gr_arg_wrapper wrapper;
 
+	signal(SIGINT, ctrl_sighandler);
 	ioctl(0, TIOCEXCL);
 
 	wrapper.version = GRADM_VERSION;
@@ -167,6 +184,9 @@ void check_acl_status(u_int16_t reqmode)
 		} else if (retval == 3) {
 			printf("The terminal you are using is unsafe.  Use another terminal.\n");
 			exit(2);
+		} else {
+			printf("The /dev/grsec device is not properly installed on your system or you are not using a grsecurity kernel.\n");
+			exit(-1);
 		}
 		break;
 	case GRADM_ENABLE:
@@ -215,6 +235,7 @@ init_variables(void)
 	dont_reduce_dirs = NULL;
 	always_reduce_dirs = NULL;
 	protected_paths = NULL;
+	read_protected_paths = NULL;
 	high_reduce_dirs = NULL;
 	high_protected_paths = NULL;
 
