@@ -123,7 +123,7 @@ static int
 is_role_dupe(struct role_acl *role, const char *rolename, const u_int16_t type)
 {
 	struct role_acl *tmp;
-	int id;
+	int id = 0;
 	int i;
 
 	if ((type & GR_ROLE_ISID) || ((type & (GR_ROLE_USER | GR_ROLE_GROUP)) && !(type & GR_ROLE_DOMAIN))) {
@@ -419,7 +419,7 @@ int
 add_globbed_object_acl(struct proc_acl *subject, char *filename,
 		  u_int32_t mode, int type, char *policy_file, unsigned long line)
 {
-	char *basepoint = gr_strdup(filename);
+	char *basepoint;
 	char *p, *p2;
 	struct file_acl *anchor;
 	struct file_acl *glob, *glob2;
@@ -428,24 +428,7 @@ add_globbed_object_acl(struct proc_acl *subject, char *filename,
 	/* one for the object itself, one for the filename */
 	num_pointers += 2;
 
-	if (!basepoint)
-		failure("gr_strdup");
-
-	/* calculate basepoint, eg basepoint of /home/ * /test is /home */
-	p = p2 = basepoint;
-	while (*p != '\0') {
-		if (*p == '/')
-			p2 = p;
-		if (*p == '?' || *p == '*' || *p == '[')
-			break;
-		p++;
-	}
-	/* if base is / */
-	if (p2 == basepoint)
-		*(p2 + 1) = '\0';
-	else
-		*p2 = '\0';
-
+	basepoint = get_anchor(filename);
 	anchor = lookup_acl_object_by_name(subject, basepoint);
 
 	if (!anchor) {
@@ -564,8 +547,6 @@ parse_homedir(char *filename)
 	return newfilename;
 }
 
-static uid_t symlink_uid;
-
 int
 add_proc_object_acl(struct proc_acl *subject, char *filename,
 		    u_int32_t mode, int type)
@@ -576,7 +557,6 @@ add_proc_object_acl(struct proc_acl *subject, char *filename,
 	struct deleted_file *dfile;
 	unsigned int file_len;
 	char *str;
-	static int link_count = 0;
 
 	if (!subject) {
 		fprintf(stderr, "Error on line %lu of %s.  Attempt to "
